@@ -4,6 +4,13 @@
 
 __all__ = ["env"]
 
+from unpythonic.dynscope import dyn, make_dynvar
+from unpythonic.misc import namelambda
+
+# Whether to automatically name lambdas saved in an env.
+# Use  with dyn.let(env_namedlambda=True): ...  to activate.
+make_dynvar(env_namedlambda=False)
+
 class env:
     """Environment for let-like constructs.
 
@@ -72,16 +79,17 @@ class env:
             raise ValueError("'{}' is not a valid identifier".format(name))
 #        value = self._wrap(name, value)  # for "e.x << value" rebind syntax.
         self._env[name] = value  # make all other attrs else live inside _env
+        if dyn.env_namedlambda:  # name lambdas saved to an unpythonic.env
+            value = namelambda(value, name)
 
     def __getattr__(self, name):
         # Block invalid names in subscripting (which redirects here).
         if not name.isidentifier():
             raise ValueError("'{}' is not a valid identifier".format(name))
-        env = self._env   # __getattr__ not called if direct attr lookup succeeds, no need for hook.
-        if name in env:
-            return env[name]
-        else:
+        e = self._env   # __getattr__ not called if direct attr lookup succeeds, no need for hook.
+        if name not in e:
             raise AttributeError("name '{:s}' is not defined".format(name))
+        return e[name]
 
     # membership test (in, not in)
     def __contains__(self, k):
@@ -90,11 +98,10 @@ class env:
     # iteration
     def __iter__(self):
         return self._env.__iter__()
-
-    def __next__(self):
-        return self._env.__next__()
+    # no __next__, iterating over dict.
 
     def items(self):
+        """Like dict.items()."""
         return self._env.items()
 
     def __len__(self):
