@@ -29,19 +29,9 @@ __all__ = ["looped", "looped_over", "breakably_looped", "breakably_looped_over"]
 
 from functools import partial
 
-# TCO implementation switchable at runtime
-import unpythonic.rc
-if unpythonic.rc._tco_impl == "exc":
-    from unpythonic.tco import SELF, jump, trampolined, _jump
-elif unpythonic.rc._tco_impl == "fast":
-    from unpythonic.fasttco import SELF, jump, trampolined, _jump
-else:
-    raise ValueError("Unknown TCO implementation '{}'".format(unpythonic.rc._tco_impl))
-
 from unpythonic.ec import call_ec
-
-# evil inspect dependency, used only to provide informative error messages.
 from unpythonic.arity import arity_includes, UnknownArity
+from unpythonic.tco import SELF, jump, trampolined, _jump
 
 def looped(body):
     """Decorator to make a functional loop and run it immediately.
@@ -64,8 +54,7 @@ def looped(body):
     but it also inserts the magic parameter ``loop``, which can only be set up
     via this mechanism.
 
-    When using ``fasttco``, **loop is a noun, not a verb**, because
-    ``unpythonic.tco.jump`` is.
+    **CAUTION**: ``loop`` is a noun, not a verb, because ``unpythonic.tco.jump`` is.
 
     Simple example::
 
@@ -214,7 +203,7 @@ def looped_over(iterable, acc=None):  # decorator factory
     but it also inserts the magic parameters ``loop``, ``x`` and ``acc``,
     which can only be set up via this mechanism.
 
-    When using ``fasttco``, **loop is a noun, not a verb**, just like ``jump``.
+    **CAUTION**: ``loop`` is a noun, not a verb, because ``unpythonic.tco.jump`` is.
 
     Examples::
 
@@ -352,9 +341,8 @@ def test():
     def s(loop, acc=0, i=0):
         if i == 10:
             return acc  # there's no "break"; loop terminates at the first normal return
-        else:
-            # same as return jump(SELF, acc+i, i+1), but sets up the "loop" arg.
-            return loop(acc + i, i + 1)
+        # same as return jump(SELF, acc+i, i+1), but sets up the "loop" arg.
+        return loop(acc + i, i + 1)
     assert s == 45
 
     # equivalent to:
@@ -362,8 +350,7 @@ def test():
     def dowork(acc=0, i=0):
         if i == 10:
             return acc
-        else:
-            return jump(dowork, acc + i, i + 1)
+        return jump(dowork, acc + i, i + 1)
     s = dowork()  # when using just @trampolined, must start the loop manually
     assert s == 45
 
@@ -426,23 +413,21 @@ def test():
     # same without using side effects - use an accumulator parameter:
     @looped
     def out(loop, i=0, acc=[]):
-        if i < 3:
-            acc.append(i)
-            return loop(i + 1)
-        else:
+        if i >= 3:
             return acc
+        acc.append(i)
+        return loop(i + 1)
     assert out == [0, 1, 2]
 
     # there's no "continue"; package your own:
     @looped
     def s(loop, acc=0, i=0):
         cont = lambda newacc=acc: loop(newacc, i + 1)
-        if i <= 4:
-            return cont()
-        elif i == 10:
+        if i == 10:
             return acc
-        else:
-            return cont(acc + i)
+        elif i <= 4:
+            return cont()
+        return cont(acc + i)
     assert s == 35
 
     # FP loop over iterable
@@ -499,10 +484,9 @@ def test():
         predicate = predicate or (lambda x: x)  # None -> truth value test
         @looped_over(iterable, acc=())
         def out(loop, x, acc):
-            if predicate(x):
-                return loop(acc + (x,))
-            else:
+            if not predicate(x):
                 return loop(acc)
+            return loop(acc + (x,))
         return out
     assert filter_fp(lambda x: x % 2 == 0, range(10)) == (0, 2, 4, 6, 8)
 
@@ -636,10 +620,9 @@ def test():
         @looped
         def s(loop, acc=0, i=0):
             cont = lambda newacc=acc: loop(newacc, i + 1)
-            if i < 10:
-                return cont(acc + i)
-            else:
+            if i >= 10:
                 return acc
+            return cont(acc + i)
         print("s is {:d}".format(s))
         return 2 * s
     assert result == 90
@@ -680,8 +663,7 @@ def test():
     def result(loop, brk, acc=0, i=0):
         if i == 10:
             return brk(acc)
-        else:
-            return loop(acc + i, i + 1)  # provide the additional parameters
+        return loop(acc + i, i + 1)  # provide the additional parameters
     assert result == 45
 
     # break, continue
