@@ -63,9 +63,18 @@ def memoize(f):
 #    return memoized
 
 make_dynvar(curry_toplevel_passthrough=False)
-make_dynvar(_curry_context=None)
+make_dynvar(_curry_context=[])
 make_dynvar(_curry_allow_uninspectable=False)  # if True, no-op if not inspectable.
-def curry(f, *args, **kwargs):
+def _currycall(f, *args, **kwargs):
+    """Co-operate with unpythonic.syntax.curry.
+
+    In a ``with curry`` block, need to call also when ``f()`` has transformed
+    to ``curry(f)``, but definitions can be curried as usual.
+
+    Hence we provide this separate mode to curry-and-call even if no args.
+    """
+    return curry(f, *args, _curry_force_call=True, **kwargs)
+def curry(f, *args, _curry_force_call=False, **kwargs):
     """Decorator: curry the function f.
 
     Essentially, the resulting function automatically chains partial application
@@ -154,7 +163,7 @@ def curry(f, *args, **kwargs):
     """
     # trivial case first: prevent stacking curried wrappers
     if iscurried(f):
-        if args or kwargs:
+        if args or kwargs or _curry_force_call:
             return f(*args, **kwargs)
         return f
     # TODO: improve: all required name-only args should be present before calling f.
@@ -164,14 +173,14 @@ def curry(f, *args, **kwargs):
     except UnknownArity:  # likely a builtin
         if not dyn._curry_allow_uninspectable:  # usual behavior
             raise
-        # co-operate with the autocurry macro; don't crash on builtins
-        if args or kwargs:
+        # co-operate with unpythonic.syntax.curry; don't crash on builtins
+        if args or kwargs or _curry_force_call:
             return f(*args, **kwargs)
         return f
     @wraps(f)
     def curried(*args, **kwargs):
         outerctx = dyn._curry_context
-        with dyn.let(_curry_context=(outerctx, f)):
+        with dyn.let(_curry_context=(outerctx + [f])):
             if len(args) < min_arity:
                 return curry(partial(f, *args, **kwargs))
             # passthrough on right, like https://github.com/Technologicat/spicy
@@ -185,7 +194,8 @@ def curry(f, *args, **kwargs):
                         now_result = curry(now_result)
                     return now_result(*later_args)
                 if not (outerctx or dyn.curry_toplevel_passthrough):
-                    raise TypeError("Top-level curry context exited with {:d} arg(s) remaining: {}".format(len(later_args), later_args))
+                    raise TypeError("Top-level curry context exited with {:d} arg(s) remaining: {}".format(len(later_args),
+                                                                                                           later_args))
                 # pass through to the curried procedure waiting in outerctx
                 # (e.g. in a curried compose chain)
                 if isinstance(now_result, tuple):
@@ -194,7 +204,7 @@ def curry(f, *args, **kwargs):
             return f(*args, **kwargs)
     curried._is_curried_function = True  # stash for detection
     # curry itself is curried: if we get args, they're the first step
-    if args or kwargs:
+    if args or kwargs or _curry_force_call:
         return curried(*args, **kwargs)
     return curried
 
@@ -423,7 +433,7 @@ def _make_compose(direction):  # "left", "right"
                 # co-operate with curry: provide a top-level curry context
                 # to allow passthrough from the function that is applied first
                 # to the function that is applied second.
-                bindings = {"_curry_context": (dyn._curry_context, composed)}
+                bindings = {"_curry_context": dyn._curry_context + [composed]}
             with dyn.let(**bindings):
                 a = g(*args)
             # we could duck-test, but this is more predictable for the user
