@@ -8,8 +8,6 @@ from functools import wraps
 
 from unpythonic.misc import call
 from unpythonic.env import env as _envcls
-
-# evil inspect dependency, used only to provide informative error messages.
 from unpythonic.arity import arity_includes, UnknownArity
 
 def let(body, **bindings):
@@ -248,9 +246,14 @@ def _let(mode, body, **bindings):
     # are then already bound when the wrappers are called.
     env = _envcls()
     for k, v in bindings.items():
+        if k in env:
+            raise AttributeError("Cannot rebind the same name '{}' in a {} initializer list".format(k, mode))
         if mode == "letrec" and callable(v):
-            if not arity_includes(v, 1):
-                raise ValueError("Arity mismatch; callable value must allow arity 1, to take in the environment.")
+            try:
+                if not arity_includes(v, 1):
+                    raise ValueError("Arity mismatch; callable value must allow arity 1, to take in the environment.")
+            except UnknownArity:  # well, we tried!
+                pass
             v = v(env)
         env[k] = v
     # decorators need just the final env; else run body now
@@ -258,26 +261,30 @@ def _let(mode, body, **bindings):
     if body:
         if not callable(body):
             raise TypeError("Expected callable body, got '{}' with value '{}'".format(type(body), body))
-        if not arity_includes(body, 1):
-            raise ValueError("Arity mismatch; body must allow arity 1, to take in the environment.")
+        try:
+            if not arity_includes(body, 1):
+                raise ValueError("Arity mismatch; body must allow arity 1, to take in the environment.")
+        except UnknownArity:  # well, we tried!
+            pass
     return env if body is None else body(env)
 
 # decorator factory: almost as fun as macros?
-def _dlet(mode, **bindings):
+# _envname is for co-operation with the dlet macro.
+def _dlet(mode, _envname="env", **bindings):
     def deco(body):
         # evaluate env only once, when the function def runs
         # (to preserve state between calls to the decorated function)
         env = _let(mode, body=None, **bindings)
         @wraps(body)
-        def decorated(*args, **kwargs):
+        def withenv(*args, **kwargs):
             kwargs_with_env = kwargs.copy()
-            kwargs_with_env["env"] = env
+            kwargs_with_env[_envname] = env
             return body(*args, **kwargs_with_env)
-        return decorated
+        return withenv
     return deco
 
-def _blet(mode, **bindings):
-    dlet_deco = _dlet(mode, **bindings)
+def _blet(mode, _envname="env", **bindings):
+    dlet_deco = _dlet(mode, _envname, **bindings)
     def deco(body):
         return call(dlet_deco(body))
     return deco
@@ -326,8 +333,25 @@ def test():
 
     uniqify_test()
 
+    # Let over lambda. The inner lambda is the definition of the function f.
+    counter = let(x=0,
+                  body=lambda e: lambda: begin(e.set("x", e.x + 1),
+                                               e.x))
+    counter()
+    counter()
+    assert counter() == 3
+
+    # let-over-def
+    @dlet(count=0)
+    def counter2(*, env=None):  # env: named argument containing the let bindings
+        env.count += 1
+        return env.count
+    counter2()
+    counter2()
+    assert(counter2() == 3)
+
     @dlet(y=23)
-    def foo(x, *, env):  # the named argument env contains the let bindings
+    def foo(x, *, env):
         return x + env.y
     assert foo(17) == 40
 
@@ -366,14 +390,6 @@ def test():
 
         assert env.x == 5
 
-    # Let over lambda. The inner lambda is the definition of the function f.
-    counter = let(x=0,
-                  body=lambda e: lambda: begin(e.set("x", e.x + 1),
-                                               e.x))
-    counter()
-    counter()
-    assert counter() == 3
-
     # https://docs.racket-lang.org/reference/let.html
     t = letrec(evenp=lambda e: lambda x: (x == 0) or e.oddp(x - 1),
                oddp=lambda e: lambda x: (x != 0) and e.evenp(x - 1),
@@ -388,12 +404,14 @@ def test():
         return evenp(42)
     assert t is True
 
+    # letrec-over-def
     @dletrec(evenp=lambda e: lambda x: (x == 0) or e.oddp(x - 1),
              oddp=lambda e: lambda x: (x != 0) and e.evenp(x - 1))
     def is_even(x, *, env):
         return env.evenp(x)
     assert is_even(23) is False
 
+    # code block with letrec
     @bletrec(evenp=lambda e: lambda x: (x == 0) or e.oddp(x - 1),
              oddp=lambda e: lambda x: (x != 0) and e.evenp(x - 1))
     def result(*, env):
@@ -403,7 +421,7 @@ def test():
     try:
         let(x=0,
             body=lambda e: e.set('y', 3))  # error, y is not defined
-    except AttributeError as err:
+    except AttributeError:
         pass
     else:
         assert False
@@ -412,7 +430,7 @@ def test():
         @blet(x=1)
         def error1(*, env):
             env.y = 2  # error, cannot introduce new bindings to a let environment
-    except AttributeError as err:
+    except AttributeError:
         pass
     else:
         assert False
