@@ -8,8 +8,6 @@ from functools import wraps
 
 from unpythonic.misc import call
 from unpythonic.env import env as _envcls
-
-# evil inspect dependency, used only to provide informative error messages.
 from unpythonic.arity import arity_includes, UnknownArity
 
 def let(bindings, body):
@@ -231,32 +229,41 @@ def _let(bindings, body, *, env=None, mode="let"):
         if body:
             if not callable(body):
                 raise TypeError("Expected callable body, got '{}' with value '{}'".format(type(body), body))
-            if not arity_includes(body, 1):
-                raise ValueError("Arity mismatch; body must allow arity 1, to take in the environment.")
+            try:
+                if not arity_includes(body, 1):
+                    raise ValueError("Arity mismatch; body must allow arity 1, to take in the environment.")
+            except UnknownArity:  # well, we tried!
+                pass
         # decorators need just the final env; else run body now
         return env if body is None else body(env)
 
     (k, v), *more = bindings
+    if k in env:
+        raise AttributeError("Cannot rebind the same name '{}' in a {} initializer list".format(k, mode))
     if mode == "letrec" and callable(v):
-        if not arity_includes(v, 1):
-            raise ValueError("Arity mismatch; callable value must allow arity 1, to take in the environment.")
+        try:
+            if not arity_includes(v, 1):
+                raise ValueError("Arity mismatch; callable value must allow arity 1, to take in the environment.")
+        except UnknownArity:  # well, we tried!
+            pass
         v = v(env)
     env[k] = v
     return _let(more, body, env=env, mode=mode)  # FP loop (without TCO)
 
-def _dlet(bindings, mode="let"):  # let and letrec decorator factory
+# _envname is for co-operation with the dlet macro.
+def _dlet(bindings, mode="let", _envname="env"):  # let and letrec decorator factory
     def deco(body):
         env = _let(bindings, body=None, mode=mode)  # set up env, don't run yet
         @wraps(body)
-        def decorated(*args, **kwargs):
+        def withenv(*args, **kwargs):
             kwargs_with_env = kwargs.copy()
-            kwargs_with_env["env"] = env
+            kwargs_with_env[_envname] = env
             return body(*args, **kwargs_with_env)
-        return decorated
+        return withenv
     return deco
 
-def _blet(bindings, mode="let"):
-    dlet_deco = _dlet(bindings, mode)
+def _blet(bindings, mode="let", _envname="env"):
+    dlet_deco = _dlet(bindings, mode, _envname)
     def deco(body):
         return call(dlet_deco(body))
     return deco
@@ -296,7 +303,7 @@ def test():
                               (x != 0) and e.evenp(x - 1))),
                lambda e:
                  e.evenp(42))
-    assert t == True
+    assert t is True
 
     f = lambda lst: letrec((("seen", set()),
                             ("see", lambda e:
@@ -363,7 +370,7 @@ def test():
         @blet((('x', 1),))
         def error1(*, env):
             env.y = 2  # error, cannot introduce new bindings to a let environment
-    except AttributeError as err:
+    except AttributeError:
         pass
     else:
         assert False
