@@ -6,11 +6,11 @@ Hashable, pickleable, hooks into the built-in reversed(), prints like in Lisps.
 """
 
 from abc import ABCMeta, abstractmethod
-from collections.abc import Sequence
 from itertools import zip_longest
 
 from unpythonic.fun import composer1i
 from unpythonic.fold import foldr, foldl
+from unpythonic.it import rev
 
 # explicit list better for tooling support
 _exports = ["cons", "nil",
@@ -36,6 +36,9 @@ __all__ = _exports
 # can correctly unpickle cons structures, which use a different nil instance
 # (from an earlier session).
 class Nil:
+    """The empty linked list.
+
+    Singleton; use the value ``nil``, don't instantiate a new one."""
     # support the iterator protocol so we can say tuple(nil) --> ()
     def __iter__(self):
         return self
@@ -86,7 +89,7 @@ class LinkedListIterator(ConsIterator):
                         raise TypeError("Not a linked list")
         super().__init__(head, walker)
 
-class ReversedLinkedListIterator(LinkedListIterator):
+class LinkedListReverseIterator(LinkedListIterator):
     """Iterator for walking a linked list backwards.
 
     Computes the reversed list at init time, so it can then be walked forward.
@@ -172,7 +175,7 @@ class cons:
         return LinkedListOrCellIterator(self)
     def __reversed__(self):
         """For lists. Caution: O(n), works by building a reversed list."""
-        return ReversedLinkedListIterator(self)
+        return LinkedListReverseIterator(self)
     def __repr__(self):
         try:  # duck test linked list (true list only, no single-cell pair)
             # listcomp, not genexpr, since we want to trigger any exceptions **now**.
@@ -278,40 +281,30 @@ def llist(iterable):
     Because cons appends to the front, this is efficient for:
 
       - ``reversed(some_linked_list)``, by just returning the already computed
-        reversed list that is internally stored by the iterator.
+        reversed list that is internally stored by the reverse-iterator.
 
       - Sequences, since they can be walked backwards; a linear walk is enough.
-        Here a sequence is defined as tuple, list, range, or any custom class
-        inheriting from ``collections.abc.Sequence``.
 
-    For a general iterable input, this costs a linear walk (forwards), plus an
-    ``lreverse`` once the list has been fully consed.
+    For a general iterable input, this costs a linear walk (forwards),
+    plus an ``lreverse``.
     """
-    if isinstance(iterable, ReversedLinkedListIterator):
+    if isinstance(iterable, LinkedListReverseIterator):
         # avoid two extra reverses by reusing the internal data.
         return iterable._data
-    if isinstance(iterable, (tuple, list, range, Sequence)):
-        return foldr(cons, nil, iterable)  # sequences can be walked backwards
-    # general iterable requires walking forwards, then reversing the result
-    # because cons appends to the front.
-    #
-    # Equivalent to lreverse(lreverse(iterable)) but this is semantically cleaner,
-    # since the original iterable is usually not a linked list. (There's no point
-    # in copying one, since they're immutable.)
-    return lreverse(foldl(cons, nil, iterable))
+    return lreverse(rev(iterable))
 
-def lreverse(l):
-    """Reverse a linked list, returning the resulting list.
+def lreverse(iterable):
+    """Reverse an iterable, loading the result into a linked list.
 
-    If you want an iterator instead, use ``reversed(l)``. The computational cost
-    is the same in both cases, O(n).
+    If you have a linked list and want an iterator instead, use ``reversed(l)``.
+    The computational cost is the same in both cases, O(n).
     """
-    return foldl(cons, nil, l)
+    return foldl(cons, nil, iterable)
 
 def lappend(*ls):
     """Append the given linked lists left-to-right."""
     def lappend_two(l1, l2):
-        return foldr(cons, l2, l1)  # this internally calls reversed() on l1
+        return foldr(cons, l2, l1)
     return foldr(lappend_two, nil, ls)
 
 def member(x, l):
@@ -320,8 +313,7 @@ def member(x, l):
     Returns:
         The matching cons cell (tail of l) if x was found; False if not.
     """
-    it = TailIterator(l)
-    for t in it:
+    for t in TailIterator(l):
         if t.car == x:
             return t
     return False
@@ -414,7 +406,7 @@ def test():
     from pickle import dumps, loads
     l = ll(1, 2, 3)
     k = loads(dumps(l))
-    # should iterate without crashing, since the nil is converted.
+    # should iterate without crashing, since the nil is refreshed.
     assert tuple(k) == (1, 2, 3)
 
     # cons structures are immutable (because cons cells are),
@@ -439,14 +431,13 @@ def test():
     assert llist(reversed(ll(1, 2, 3))) == ll(3, 2, 1)
     assert foldl(cons, nil, ll(1, 2, 3)) == ll(3, 2, 1)
 
-    # foldr (as implemented in unpythonic) requires support for __reversed__,
-    # since a linked list is an iterable, not a sequence.
-    assert foldr(cons, nil, ll(1, 2, 3)) == ll(1, 2, 3)
-
     # implementation detail to avoid extra reverses
     r = reversed(ll(1, 2, 3))   # an iterator that internally builds the reversed list...
     assert llist(r) is r._data  # ...which llist should just grab
 
+    # foldr implicitly reverses the input
+    assert foldr(cons, nil, ll(1, 2, 3)) == ll(1, 2, 3)
+
     print("All tests PASSED")
 
 if __name__ == '__main__':
